// vector1.cpp		By: Aiman Hanna  1993 - 2004

// C provided arrays as a container type. However arrays support the mechanics of a 
// container without supporting the semantics of the container abstraction. Prior to 
// Standard C++, programmers had to build their own classes to achieve this. With 
// Standard C++, an array class is a part of the Standard. However it is not called 
// an array; it is called a vector.
//
// A vector is in fact a class template; for example 
//		vector<int> ivec(100) 
// creates a vector of 100 integers. 
// 
// There are differences however between arrays and vectors. The first primary 
// difference is that vectors can grow dynamically at runtime. The second is a 
// design-related issue. Instead of providing a large set of methods for the vector 
// class (such as find(), sort(), ..etc), the vector class provides a minimal  
// set (such as size(), empty(),...). 

// Finally, to define a vector we must include the associated header file; that is
//  #include <vector> 	// see the note below
//
// Note about the #include directive
// =================================
// 
// The #include directive for a C header file does not have the same effect depending 
// on whether the C name or the C++ name is used. 
// for example, #include <cassert> causes the content of the cassert to be read into 
// the text file that has the #include. However, because all C++ library names are 
// declared in namespace std, the name assert() will not be visible in a program text 
// file unless it is explicitly made available with the "using namespace std" directive.
// With a #include directive that uses the C header files (that is #include <assert.h>), 
// the name assert can be used directly in the text file that has the #include. 
//
//		Key Points: 1) vectors 
//					2) include directive (difference between C & C++)
//					3) using namespace std

#include <iostream.h>

#include <vector> 
using namespace std;

int main()
{
	const int size = 10;
	const int initValue = 50;
	
	
	// The following shows different ways to create vectors

	vector<int> v1;		// An empty vector

	vector<int> v2(size);	// An array of 10 integers; all initialized to 0

	vector<int> v3(size, initValue);	// An array of 10 integers; all initialized to 50

	vector<int> v4( v3 );	// v4 is a copy of v3
	
	int Arr[7] = {0, 5, 10, 15, 20, 25, 30};
	vector<int> v5( Arr, Arr+4 );	// Initialize v5 to Arr; only the first 4 values are copied
	
	vector<int> v6( Arr, Arr+7 );	// Initialize v6 to Arr; all values are copied

	vector<int> v7( Arr, Arr+12 );	// Initialize v7 to Arr; this is too bad, why???

	cout << "Contents of vector v1\n";
	cout << "=====================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for (int i = 0; i < v1.size(); i++)
	{
		cout << v1[i] << "  ";
	}

	cout << "\n\nContents of vector v2\n";
	cout << "=====================\n";
	cout << "Current size of v2 is: " << v2.size() << endl; 
	for (i = 0; i < v2.size(); i++)
	{
		cout << v2[i] << "  ";
	}

	cout << "\n\nContents of vector v3\n";
	cout << "=====================\n";
	cout << "Current size of v3 is: " << v3.size() << endl; 
	for (i = 0; i < v3.size(); i++)
	{
		cout << v3[i] << "  ";
	}

	cout << "\n\nContents of vector v4\n";
	cout << "=====================\n";
	cout << "Current size of v4 is: " << v4.size() << endl; 
	for (i = 0; i < v4.size(); i++)
	{
		cout << v4[i] << "  ";
	}

	cout << "\n\nContents of vector v5\n";
	cout << "=====================\n";
	cout << "Current size of v5 is: " << v5.size() << endl; 
	for (i = 0; i < v5.size(); i++)
	{
		cout << v5[i] << "  ";
	}
	
	cout << "\n\nContents of vector v6\n";
	cout << "=====================\n";
	cout << "Current size of v6 is: " << v6.size() << endl; 
	for (i = 0; i < v6.size(); i++)
	{
		cout << v6[i] << "  ";
	}

	cout << "\n\nContents of vector v7\n";
	cout << "=====================\n";
	cout << "Current size of v7 is: " << v7.size() << endl; 
	for (i = 0; i < v7.size(); i++)
	{
		cout << v7[i] << "  ";
	}

	// Append some values to v1, v2, v3 & v7. More memory will be allocated if needed
	v1.push_back(90);
	v2.push_back(65);
	v3.push_back(86);
	v7.push_back(109);

	cout << "\n\nContents of vector v1 now\n";
	cout << "========================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for ( i = 0; i < v1.size(); i++)
	{
		cout << v1[i] << "  ";
	}

	cout << "\n\nContents of vector v2\n";
	cout << "=====================\n";
	cout << "Current size of v2 is: " << v2.size() << endl; 
	for (i = 0; i < v2.size(); i++)
	{
		cout << v2[i] << "  ";
	}

	cout << "\n\nContents of vector v3 now\n";
	cout << "========================\n";
	cout << "Current size of v3 is: " << v3.size() << endl; 
	for (i = 0; i < v3.size(); i++)
	{
		cout << v3[i] << "  ";
	}

	cout << "\n\nContents of vector v7\n";
	cout << "=====================\n";
	cout << "Current size of v7 is: " << v7.size() << endl; 
	for (i = 0; i < v7.size(); i++)
	{
		cout << v7[i] << "  ";
	}

	
	// remove (pop) one element from v1, and two elements from v3 then display them only if they are not empty	
	v1.pop_back();
	v3.pop_back();
	v3.pop_back();

	if(!v1.empty())
	{
	cout << "\n\nContents of vector v1 now\n";
	cout << "========================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for ( i = 0; i < v1.size(); i++)
		{
			cout << v1[i] << "  ";
		}

	}
	

	if (!v3.empty())
	{
	cout << "\n\nContents of vector v3 now\n";
	cout << "========================\n";
	cout << "Current size of v3 is: " << v3.size() << endl; 
	for (i = 0; i < v3.size(); i++)
		{
			cout << v3[i] << "  ";
		}

	}

	cout << "\n\nValue at index 3 (4th element) of v6 is: " << v6.at(3) << endl;
	
	// Clear the contents of v6; this will remove all elements
	v6.clear();

	cout << "\n\nContents of vector v6 now\n";
	cout << "========================\n";
	cout << "Current size of v6 is: " << v6.size() << endl; 
	for (i = 0; i < v6.size(); i++)
	{
		cout << v6[i] << "  ";
	}

	
	cout << endl;
	return 0;
}

/* The output
Contents of vector v1
=====================
Current size of v1 is: 0


Contents of vector v2
=====================
Current size of v2 is: 10
0  0  0  0  0  0  0  0  0  0

Contents of vector v3
=====================
Current size of v3 is: 10
50  50  50  50  50  50  50  50  50  50

Contents of vector v4
=====================
Current size of v4 is: 10
50  50  50  50  50  50  50  50  50  50

Contents of vector v5
=====================
Current size of v5 is: 4
0  5  10  15

Contents of vector v6
=====================
Current size of v6 is: 7
0  5  10  15  20  25  30

Contents of vector v7
=====================
Current size of v7 is: 12
0  5  10  15  20  25  30  -858993460  4464208  4464248  4464248  -858993460

Contents of vector v1 now
========================
Current size of v1 is: 1
90

Contents of vector v2
=====================
Current size of v2 is: 11
0  0  0  0  0  0  0  0  0  0  65

Contents of vector v3 now
========================
Current size of v3 is: 11
50  50  50  50  50  50  50  50  50  50  86

Contents of vector v7
=====================
Current size of v7 is: 13
0  5  10  15  20  25  30  -858993460  4464208  4464248  4464248  -858993460  109

Contents of vector v3 now
========================
Current size of v3 is: 9
50  50  50  50  50  50  50  50  50

Value at index 3 (4th element) of v6 is: 15


Contents of vector v6 now
========================
Current size of v6 is: 0

*/