// vector3.cpp		By: Aiman Hanna  1993 - 2004

// The operations that can be applied on vectors do vary, however they are not provided 
// as member functions of the vector class. Instead, they are defined as an independent
// set of generic algorithms provided by the standard library. 
// This program illustrates some of these algorithms. In specific, the program 
// illustrates some of the available algorithms for vector sorting, searching, reversing and few others.
// Finally, to use these algorithms, their associated header file must be included. 
//
//		Key Points: 1) Algorithms of vector (such as searching, sorting, reversing, ..etc) 

#include <iostream>		// No .h; we can do so since we have "using namespace std;
#include <vector> 
#include <algorithm> 

using namespace std;

int main()
{
	
	int Arr[10] = {0, 25, 9, 15, 88, 3, 25, 88, 221, 156};
	vector<int> v1( Arr, Arr+10 );	// Initialize v1 to Arr	
	vector<int> v2( Arr, Arr+10 );	// Initialize v2 to Arr


	vector<int>::iterator iter1 = v1.begin();

	vector<int>::iterator iter2, iter22;


	int val;
	

	cout << "Contents of vector v1\n";
	cout << "=====================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for ( iter1; iter1 != v1.end(); iter1++)
	{
		cout << *iter1 << "  ";
	}

	cout << "\n\nContents of vector v2\n";
	cout << "=====================\n";
	cout << "Current size of v2 is: " << v2.size() << endl; 
	for ( iter2 = v2.begin(); iter2 != v2.end(); iter2++)
	{
		cout << *iter2 << "  ";
	}

	cout << "\n\nEnter the value you want to search for in vector v1: ";
	cin >> val;

	vector<int>::iterator found;
	vector<int>::iterator last;
	
	found = find(v1.begin(), v1.end(), val);
	if(found != v1.end())
	{
		cout << "Value was found in vector v1 " << count(v1.begin(), v1.end(), val) << " time(s)\n";
		cout << "Value first occurred at index " << v1.size() - ( v1.end() - found ) << endl;
	}
	else
		cout << "Value was not found in vector v1\n";

	// Sort entire vector v2
	sort(v2.begin(), v2.end());

	cout << "\n\nContents of vector v2 after sorting\n";
	cout << "====================================\n";
	cout << "Current size of v2 is: " << v2.size() << endl; 
	for ( iter2 = v2.begin(); iter2 != v2.end(); iter2++)
	{
		cout << *iter2 << "  ";
	}


	// partially sort vector v1, first 6 elements

	sort(v1.begin(), v1.end() - (v1.size() - 6));
	cout << "\n\nContents of vector v1 after partial sorting\n";
	cout << "==========================================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for ( iter1 = v1.begin(); iter1 != v1.end(); iter1++)
	{
		cout << *iter1 << "  ";
	}


	// Reverse vector v1
	reverse(v1.begin(), v1.end());
	cout << "\n\nContents of vector v1 after being reverses\n";
	cout << "===========================================\n";
	cout << "Current size of v1 is: " << v1.size() << endl; 
	for ( iter1 = v1.begin(); iter1 != v1.end(); iter1++)
	{
		cout << *iter1 << "  ";
	}

	// Remove all elements with values 25 from v2
	iter2 = v2.begin();
	iter22 = v2.end();

	last = remove(iter2, iter22, 25);	// same as: last = remove(v2.begin(), ve.end(), 25);

	// last now points to the last valid element in the vector
	// Notice the incorrect output now if v2.end() is used.  

	cout << "\n\nContents of vector v2 after removing 25 " << v2.end() - last << " time(s). Is this correct???\n";
	cout << "=====================================================================\n";
	cout << "Current size of v2 is: " << v2.size() << endl; 
	for ( iter2 = v2.begin(); iter2 != v2.end(); iter2++)
	{
		cout << *iter2 << "  ";
	}



	cout << "\n\nValid Contents of vector v2 after removing 25 " << v2.end() - last << " time(s)\n";
	cout << "========================================================\n";
	cout << "Current size of v2 is: " << v2.size() - (v2.end() - last) << endl; 
	for ( iter2 = v2.begin(); iter2 < last; iter2++)
	{
		cout << *iter2 << "  ";
	}

	cout << endl;
	return(0);

}

/* The output
Contents of vector v1
=====================
Current size of v1 is: 10
0  25  9  15  88  3  25  88  221  156

Contents of vector v2
=====================
Current size of v2 is: 10
0  25  9  15  88  3  25  88  221  156

Enter the value you want to search for in vector v1: 88
Value was found in vector v1 2 time(s)
Value first occurred at index 4


Contents of vector v2 after sorting
====================================
Current size of v2 is: 10
0  3  9  15  25  25  88  88  156  221

Contents of vector v1 after partial sorting
==========================================
Current size of v1 is: 10
0  3  9  15  25  88  25  88  221  156

Contents of vector v1 after being reverses
===========================================
Current size of v1 is: 10
156  221  88  25  88  25  15  9  3  0

Contents of vector v2 after removing 25 2 time(s). Is this correct???
=====================================================================
Current size of v2 is: 10
0  3  9  15  88  88  156  221  156  221

Valid Contents of vector v2 after removing 25 2 time(s)
========================================================
Current size of v2 is: 8
0  3  9  15  88  88  156  221
*/

/* Run again
Contents of vector v1
=====================
Current size of v1 is: 10
0  25  9  15  88  3  25  88  221  156

Contents of vector v2
=====================
Current size of v2 is: 10
0  25  9  15  88  3  25  88  221  156

Enter the value you want to search for in vector v1: 221
Value was found in vector v1 1 time(s)
Value first occurred at index 8


Contents of vector v2 after sorting
====================================
Current size of v2 is: 10
0  3  9  15  25  25  88  88  156  221

Contents of vector v1 after partial sorting
==========================================
Current size of v1 is: 10
0  3  9  15  25  88  25  88  221  156

Contents of vector v1 after being reverses
===========================================
Current size of v1 is: 10
156  221  88  25  88  25  15  9  3  0

Contents of vector v2 after removing 25 2 time(s). Is this correct???
=====================================================================
Current size of v2 is: 10
0  3  9  15  88  88  156  221  156  221

Valid Contents of vector v2 after removing 25 2 time(s)
========================================================
Current size of v2 is: 8
0  3  9  15  88  88  156  221
*/